/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to You under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.apache.webbeans.decorator;

import javax.interceptor.AroundInvoke;
import javax.interceptor.Interceptor;
import javax.interceptor.InvocationContext;

@Interceptor
public class WebBeansDecoratorInterceptor
{

    private DelegateHandler delegate;
    private Object instance;

    public WebBeansDecoratorInterceptor(DelegateHandler delegate, Object instance)
    {
        this.delegate = delegate;
        this.instance = instance;
    }

    /**
     * Acts as the entry point into the Decorator stack when this Interceptor is
     * added to the end of the InterceptorStack.
     * 
     * @throws Exception
     */
    @AroundInvoke
    public Object invokeDecorators(InvocationContext ctx) throws Exception
    {
        return delegate.invoke(instance, ctx.getMethod(), ctx.getMethod(), ctx.getParameters());
    }

}
